-- Class.lua
-- Compatible with Lua 5.1 (not 5.0).
--
-- Modified to use a metatable to reference base objects instead
-- or performing shallow copies. This means smaller classes that
-- are a bit more dynamic at the expence of some performance.

-- if class is already defined just return
if (_G.class  ~= nil) then
    return;
end

_G.class = function( base )
	if ( base ~= nil ) then
		local baseType = type( base );

		if ( baseType ~= 'table' ) then
			error( "Base class is not a table. Base class was a " .. baseType );
		end
	end

	-- The class definition table. Contains all of the actual class
	-- methods, constructor, and an IsA function automatically
	-- provided.
	local c  = { };

	c.base = base;
	c.IsA = function( self, klass )
		local m = getmetatable( self );
		
		while m do 
			if m == klass then
				return true;
			end
			
			m = m.base;
		end
		
		return false;
	end

	-- The class definition metatable. This is used to hook up
	-- base classes and then to register a call handler that is
	-- used to construct new instances of the class.
	local mt = { };
	mt.__index = base;

	mt.__call = function( class_tbl, ... )
		if ( class_tbl.Abstract ) then
			error "Abstract class cannot be constructed.";
		end

		-- Create the new class instance.
		local instance = { };
		setmetatable( instance, { __index = c } );
		
		-- Invoke the constructor if it exists.
		if ( ( instance.Constructor ~= nil ) and ( type( instance.Constructor ) == 'function' ) ) then
			instance:Constructor( ... );
		end
		
		return instance;
	end

	setmetatable( c, mt );
	return c;
end








-- safely add a listener without overwriting any existing ones
function AddListener(object, event, listener)
    if (object[event] == nil) then
        object[event] = listener;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], listener);
        else
            object[event] = {object[event], listener};
        end
    end
end

-- safetly remove a listener without clobbering any extras
function RemoveListener(object, event, listener)
    if (object[event] == listener) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == listener) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

-- safetly execute a listener whether it be an array of functions or a single one
function ExecuteListener(object, event, args)
    if (type(object[event]) == "function") then
        object[event](args);
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (type(object[event][i]) == "function") then
                    object[event][i](args);
                end
            end
        end
    end
end

-- common time formatters, values passed are in seconds
TimeFormat = {};
TimeFormat.LITERAL = function(value)
    return value;
end
TimeFormat.TRUNCATED = function(value)
    if (value >= 3600) then
        if value % 3600 == 0 then
            value = ("%dh"):format(value / 3600)
        else
            value = ("%.1fh"):format(value / 3600)
        end
    elseif (value >= 60) then
        if value % 60 == 0 then
            value = ("%dm"):format(value / 60)
        else
            value = ("%.1fm"):format(value / 60)
        end
    else
        value = ("%ds"):format(value);
    end
    return value;
end
TimeFormat.CLOCK = function(value)
    if (value >= 3600) then
        local sec = math.fmod(value, 3600) / 60;
        value = ("%d:%02d:%02d"):format(value / 3600, value % 3600 / 60, value % 60);
    elseif (value >= 60) then
        value = ("%d:%02d"):format(value / 60, value % 60);
    else
        value = ("0:%02d"):format(value);
    end
    return value;
end